@@ -13,8 +13,6 @@ struct bios_dev_name_opts {
 	int sortroutine;
 	int namingpolicy;
 	const char *prefix;
-	unsigned int verbose:1;
-	unsigned int show_all:1;
 	unsigned int debug:1;
 	unsigned int interface:1;
 };
