@@ -21,6 +21,8 @@
 #include "dmidecode/dmidecode.h"
 #include "pirq.h"
 
+extern int is_valid_smbios;
+
 #ifndef PCI_CB_CAPABILITY_LIST
 #define PCI_CB_CAPABILITY_LIST  0x14
 #endif
@@ -191,9 +193,24 @@ static int read_pci_vpd(struct libbiosdevname_state *state, struct pci_device *p
 static void set_pci_vpd_instance(struct libbiosdevname_state *state)
 {
 	struct pci_device *dev, *dev2;
+	int fd;
+	char sys_vendor[10] = {0};
+
+	/* Read VPD-R on Dell systems only */
+	if ((fd = open("/sys/devices/virtual/dmi/id/sys_vendor", O_RDONLY)) >= 0) {
+		if (read(fd, sys_vendor, 9) != 9)
+			return;
+		if (strncmp(sys_vendor, "Dell Inc.", 9)) 
+			return;
+	} else
+		return;
 
 	/* Read VPD information for each device */
 	list_for_each_entry(dev, &state->pci_devices, node) {
+		/* RedHat bugzilla 801885, 789635, 781572 */
+		if (dev->pci_dev->vendor_id == 0x1969 ||
+		    dev->pci_dev->vendor_id == 0x168c)
+			continue;
 		read_pci_vpd(state, dev);
 	}
 
@@ -498,7 +515,7 @@ static void dev_to_slot(struct libbiosdevname_state *state, struct pci_device *d
 	int slot;
 	do {
 		slot = pci_dev_to_slot(state, d);
-		if (slot == PHYSICAL_SLOT_UNKNOWN)
+		if (slot == PHYSICAL_SLOT_UNKNOWN && is_valid_smbios)
 		  	slot = pcie_get_slot(state, d);
 		if (slot == PHYSICAL_SLOT_UNKNOWN)
 			slot = pirq_dev_to_slot(state, d);
@@ -601,6 +618,24 @@ void free_pci_devices(struct libbiosdevname_state *state)
 	}
 }
 
+int addslot(struct libbiosdevname_state *state, int slot)
+{
+	struct slotlist *s;
+
+	list_for_each_entry(s, &state->slots, node) {
+		if (s->slot == slot) {
+			return ++s->count;
+		}
+	}
+	s = malloc(sizeof(*s));
+	INIT_LIST_HEAD(&s->node);
+	s->slot = slot;
+	s->count = 0;
+	list_add(&s->node, &state->slots);
+
+	return ++s->count;
+}
+
 static void set_pci_slots(struct libbiosdevname_state *state)
 {
 	struct pci_device *dev;
@@ -608,53 +643,19 @@ static void set_pci_slots(struct libbiosdevname_state *state)
 	list_for_each_entry(dev, &state->pci_devices, node) {
 		dev_to_slot(state, dev);
 	}
-}
 
-
-static int set_pci_slot_index(struct libbiosdevname_state *state)
-{
-	struct pci_device *pcidev;
-	int prevslot=-1;
-	int index=1;
-
-	/* only iterate over the PCI devices, because the bios_device list may be incomplete due to renames happening in parallel */
-	list_for_each_entry(pcidev, &state->pci_devices, node) {
-		if (pcidev->physical_slot == 0) /* skip embedded devices */
+	/* Get mapping for each slot */
+	list_for_each_entry(dev, &state->pci_devices, node) {
+		if (dev->is_sriov_virtual_function || !is_pci_network(dev) || dev->vpd_port != INT_MAX) {
 			continue;
-		if (!is_pci_network(pcidev)) /* only look at PCI network devices */
-			continue;
-		if (pcidev->is_sriov_virtual_function) /* skip sriov VFs, they're handled later */
-			continue;
-		if (pcidev->physical_slot != prevslot) {
-			index=1;
-			prevslot = pcidev->physical_slot;
 		}
-		else
-			index++;
-		pcidev->index_in_slot = index;
+		if (dev->physical_slot == 0) {
+			dev->embedded_index_valid = 1;
+			dev->embedded_index = addslot(state, 0);
+		} else if (dev->physical_slot != PHYSICAL_SLOT_UNKNOWN) {
+			dev->index_in_slot = addslot(state, dev->physical_slot);
+		}
 	}
-	return 0;
-}
-
-static int set_embedded_index(struct libbiosdevname_state *state)
-{
-	struct pci_device *pcidev;
-	int index=1;
-
-	list_for_each_entry(pcidev, &state->pci_devices, node) {
-		if (pcidev->physical_slot != 0) /* skip non-embedded devices */
-			continue;
-		if (!is_pci_network(pcidev)) /* only look at PCI network devices */
-			continue;
-		if (pcidev->is_sriov_virtual_function) /* skip sriov VFs, they're handled later */
-			continue;
-		if (pcidev->vpd_port != INT_MAX)
-			continue;
-		pcidev->embedded_index = index;
-		pcidev->embedded_index_valid = 1;
-		index++;
-	}
-	return 0;
 }
 
 static void set_sriov_pf_vf(struct libbiosdevname_state *state)
@@ -737,8 +738,6 @@ int get_pci_devices(struct libbiosdevname_state *state)
 	sort_device_list(state);
 	set_pci_vpd_instance(state);
 	set_pci_slots(state);
-	set_embedded_index(state);
-	set_pci_slot_index(state);
 	set_sriov_pf_vf(state);
 
 	return rc;
